@@ -1,3 +1,6 @@
+# -*- coding: utf-8 -*-
+
+import io
 import os
 import pytest
 import zipfile
@@ -69,6 +72,25 @@ class TestNbGraderZipCollect(BaseTestApp):
         assert os.path.isdir(extracted_dir)
         assert len(os.listdir(extracted_dir)) == 1
 
+    def test_extract_single_notebook_f(self, course_dir, archive_dir):
+        extracted_dir = join(archive_dir, "..", "extracted")
+        self._make_notebook(archive_dir,
+            'ps1', 'hacker', '2016-01-30-15-30-10', 'problem1')
+
+        run_nbgrader(["zip_collect", "ps1"])
+        assert os.path.isdir(extracted_dir)
+        assert len(os.listdir(extracted_dir)) == 1
+
+        # Run again should fail
+        run_nbgrader(["zip_collect", "ps1"], retcode=1)
+        assert os.path.isdir(extracted_dir)
+        assert len(os.listdir(extracted_dir)) == 1
+
+        # Run again with --force flag should pass
+        run_nbgrader(["zip_collect", "-f", "ps1"])
+        assert os.path.isdir(extracted_dir)
+        assert len(os.listdir(extracted_dir)) == 1
+
     def test_extract_sub_dir_single_notebook(self, course_dir, archive_dir):
         extracted_dir = join(archive_dir, "..", "extracted")
         self._make_notebook(join(archive_dir, 'hacker'),
@@ -346,7 +368,7 @@ class TestNbGraderZipCollect(BaseTestApp):
                 """
             ))
 
-        run_nbgrader(["assign", "ps1"])
+        run_nbgrader(["generate_assignment", "ps1"])
         self._make_notebook(archive_dir,
             'ps1', 'hacker', '2016-01-30-15-30-10', 'myproblem1')
 
@@ -477,11 +499,11 @@ class TestNbGraderZipCollect(BaseTestApp):
 
         # submissions are sorted so a before b
         os.makedirs(join(archive_dir, 'ps1_hacker_a_2017-01-30-15-30-10'))
-        with open(join(archive_dir, 'ps1_hacker_a_2017-01-30-15-30-10', 'problem1.ipynb'), 'w') as fh:
-            fh.write('')
+        with io.open(join(archive_dir, 'ps1_hacker_a_2017-01-30-15-30-10', 'problem1.ipynb'), mode='w', encoding='utf-8') as fh:
+            fh.write(u'')
         os.makedirs(join(archive_dir, 'ps1_hacker_b_2016-01-30-15-30-10'))
-        with open(join(archive_dir, 'ps1_hacker_b_2016-01-30-15-30-10', 'problem1.ipynb'), 'w') as fh:
-            fh.write('')
+        with io.open(join(archive_dir, 'ps1_hacker_b_2016-01-30-15-30-10', 'problem1.ipynb'), mode='w', encoding='utf-8') as fh:
+            fh.write(u'')
 
         with open('plugin_five.py', 'w') as fh:
             fh.write(dedent(
@@ -529,11 +551,11 @@ class TestNbGraderZipCollect(BaseTestApp):
 
         # submissions are sorted so a before b
         os.makedirs(join(archive_dir, 'ps1_hacker_a_2016-01-30-15-30-10'))
-        with open(join(archive_dir, 'ps1_hacker_a_2016-01-30-15-30-10', 'problem1.ipynb'), 'w') as fh:
-            fh.write('')
+        with io.open(join(archive_dir, 'ps1_hacker_a_2016-01-30-15-30-10', 'problem1.ipynb'), mode='w', encoding='utf-8') as fh:
+            fh.write(u'')
         os.makedirs(join(archive_dir, 'ps1_hacker_b_2017-01-30-15-30-10'))
-        with open(join(archive_dir, 'ps1_hacker_b_2017-01-30-15-30-10', 'problem1.ipynb'), 'w') as fh:
-            fh.write('')
+        with io.open(join(archive_dir, 'ps1_hacker_b_2017-01-30-15-30-10', 'problem1.ipynb'), mode='w', encoding='utf-8') as fh:
+            fh.write(u'')
 
         with open('plugin_six.py', 'w') as fh:
             fh.write(dedent(
@@ -580,8 +602,8 @@ class TestNbGraderZipCollect(BaseTestApp):
         submitted_dir = join(course_dir, "submitted")
 
         os.makedirs(join(archive_dir, 'ps1_hacker'))
-        with open(join(archive_dir, 'ps1_hacker', 'problem1.ipynb'), 'w') as fh:
-            fh.write('')
+        with io.open(join(archive_dir, 'ps1_hacker', 'problem1.ipynb'), mode='w', encoding='utf-8') as fh:
+            fh.write(u'')
         with open(join(archive_dir, 'ps1_hacker', 'timestamp.txt'), 'w') as fh:
             fh.write('foo')
 
@@ -613,8 +635,8 @@ class TestNbGraderZipCollect(BaseTestApp):
         submitted_dir = join(course_dir, "submitted")
 
         os.makedirs(join(archive_dir, 'ps1_hacker', 'files'))
-        with open(join(archive_dir, 'ps1_hacker', 'files', 'problem1.ipynb'), 'w') as fh:
-            fh.write('')
+        with io.open(join(archive_dir, 'ps1_hacker', 'files', 'problem1.ipynb'), mode='w', encoding='utf-8') as fh:
+            fh.write(u'')
         with open(join(archive_dir, 'ps1_hacker', 'timestamp.txt'), 'w') as fh:
             fh.write('foo')
 
@@ -646,12 +668,12 @@ class TestNbGraderZipCollect(BaseTestApp):
         submitted_dir = join(course_dir, "submitted")
 
         os.makedirs(join(archive_dir, 'ps1_hacker_01', 'files'))
-        with open(join(archive_dir, 'ps1_hacker_01', 'files', 'problem1.ipynb'), 'w') as fh:
-            fh.write('')
+        with io.open(join(archive_dir, 'ps1_hacker_01', 'files', 'problem1.ipynb'), mode='w', encoding='utf-8') as fh:
+            fh.write(u'')
 
         os.makedirs(join(archive_dir, 'ps1_hacker_02', 'files'))
-        with open(join(archive_dir, 'ps1_hacker_02', 'files', 'problem1.ipynb'), 'w') as fh:
-            fh.write('')
+        with io.open(join(archive_dir, 'ps1_hacker_02', 'files', 'problem1.ipynb'), mode='w', encoding='utf-8') as fh:
+            fh.write(u'')
 
         with open("nbgrader_config.py", "a") as fh:
             fh.write(dedent(
